@@ -37,7 +37,13 @@
 
 #ifdef MAEMO
 #include <libosso.h>
+
+#if GTK_CHECK_VERSION(2,10,0)
 #include <tablet-browser-interface.h>
+#include <conbtdialogs-dbus.h>
+#include <libmodest-dbus-client/libmodest-dbus-client.h>
+#endif
+
 #endif
 
 
@@ -45,7 +51,9 @@ typedef struct {
    GtkWidget            *widget;
    gint                  index;
    const gchar          *title;
+#if GTK_CHECK_VERSION(2,10,0)
    GtkAssistantPageType  type;
+#endif
    gboolean              complete;
 } PageInfo;
 
@@ -54,6 +62,7 @@ static GtkWidget *tablel = NULL;
 static int        car_inserted = FALSE;
 static GtkWidget *selector2;
 static GtkWidget *selector3;
+static GtkWidget *selector4;
 static GtkWidget *lastMPG;
 static GtkWidget *lifeMPG;
 static GtkWidget *milesMonth;
@@ -72,23 +81,25 @@ static GtkWidget *assist_export;
 static GtkWidget *assist_apply;
 static GtkWidget *export_file_text;
 static GtkWidget *import_file_text;
+#ifdef MAEMO
+static GtkWidget *email_but;
+#endif
 static int        running_export = 0;
 static int        running_import = 0;
 
 static void car_selector_clicked_cb(GtkWidget* w, gpointer data);
 static void car_units_selected_clicked_cb(GtkWidget* w, gpointer data);
+static void car_units_entry_selected_clicked_cb(GtkWidget* w, gpointer data);
 static void car_cost_units_clicked_cb(GtkWidget* w, gpointer data);
 static void car_new_clicked_cb(GtkWidget* w, gpointer data);
 static void car_delete_clicked_cb(GtkWidget* w, gpointer data);
-static void car_import_clicked_cb(GtkWidget* w, gpointer data);
-static void car_export_clicked_cb(GtkWidget* w, gpointer data);
 static void car_tab_assist_cancel(GtkWidget* w, gpointer data);
 static void car_tab_assist_export_cancel(GtkWidget* w, gpointer data);
 static void car_tab_assist_apply(GtkWidget* w, gpointer data);
 static void car_tab_assist_export_apply(GtkWidget* w, gpointer data);
+static void car_import_select_file(GtkWidget* w, gpointer data);
 static void car_import_data_now(GtkWidget* w, gpointer data);
 static void car_export_select_file(GtkWidget* w, gpointer data);
-static void car_import_select_file(GtkWidget* w, gpointer data);
 
 void car_tab_layout(GtkWidget *window)
 {
@@ -96,9 +107,9 @@ void car_tab_layout(GtkWidget *window)
    GtkWidget *vboxl;
    GtkWidget *vboxr;
    GtkWidget *vsep;
-   GtkWidget *label1, *label2, *label6;
+   GtkWidget *label1, *label2, *label6, *label7;
    GtkWidget *car_entry;
-   GtkWidget *button1, *button2, *button3, *button4;
+   GtkWidget *button1, *button2;
    GtkTextView *text_box;
    GtkWidget *tabler;
    GtkWidget *sw;
@@ -147,6 +158,10 @@ void car_tab_layout(GtkWidget *window)
                      NULL);
    gtk_table_attach(GTK_TABLE(tabler), button2, 0, 4, 2, 3, GTK_FILL, GTK_FILL, 2, 2);
 
+#ifndef MAEMO
+#if GTK_CHECK_VERSION(2,10,0)
+   GtkWidget  *button3, *button4;
+
    button3= gtk_button_new_with_label("Import Mileage Data");
    gtk_signal_connect(GTK_OBJECT(button3),
                      "clicked",
@@ -160,54 +175,73 @@ void car_tab_layout(GtkWidget *window)
                      GTK_SIGNAL_FUNC(car_export_clicked_cb),
                      NULL);
    gtk_table_attach(GTK_TABLE(tabler), button4, 0, 4, 4, 5, GTK_FILL, GTK_FILL, 2, 2);
+#endif
+#endif
 
    label2 = gtk_label_new("Display Units");
    gtk_table_attach(GTK_TABLE(tablel), label2, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 2, 2);
 
    selector2 = gtk_combo_box_new_text();
-   gtk_combo_box_insert_text((GtkComboBox *)selector2, 0, "MPG");
-   gtk_combo_box_insert_text((GtkComboBox *)selector2, 1, "L/100km");
-   gtk_combo_box_insert_text((GtkComboBox *)selector2, 2, "L/10km");
-   gtk_combo_box_insert_text((GtkComboBox *)selector2, 3, "Miles/L");
-   gtk_combo_box_insert_text((GtkComboBox *)selector2, 4, "km/L");
+   gtk_combo_box_insert_text((GtkComboBox *)selector2, CAR_MPG_US_INDEX, CAR_MPG_US_TEXT);
+   gtk_combo_box_insert_text((GtkComboBox *)selector2, CAR_MPG_IMP_INDEX, CAR_MPG_IMP_TEXT);
+   gtk_combo_box_insert_text((GtkComboBox *)selector2, CAR_LP100KM_INDEX, CAR_LP100KM_TEXT);
+   gtk_combo_box_insert_text((GtkComboBox *)selector2, CAR_LP10KM_INDEX, CAR_LP10KM_TEXT);
+   gtk_combo_box_insert_text((GtkComboBox *)selector2, CAR_MPL_INDEX, CAR_MPL_TEXT);
+   gtk_combo_box_insert_text((GtkComboBox *)selector2, CAR_KMPL_INDEX, CAR_KMPL_TEXT);
    gtk_table_attach(GTK_TABLE(tablel), selector2, 1, 4, 1, 2, GTK_FILL, GTK_FILL, 2, 2);
    gtk_signal_connect(GTK_OBJECT(selector2),
                      "changed",
                      GTK_SIGNAL_FUNC(car_units_selected_clicked_cb),
                      selector2);
 
+   label7 = gtk_label_new("Entry Units");
+   gtk_table_attach(GTK_TABLE(tablel), label7, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 2, 2);
+
+   selector4 = gtk_combo_box_new_text();
+   gtk_combo_box_insert_text((GtkComboBox *)selector4, CAR_MPG_US_INDEX, CAR_MPG_US_TEXT);
+   gtk_combo_box_insert_text((GtkComboBox *)selector4, CAR_MPG_IMP_INDEX, CAR_MPG_IMP_TEXT);
+   gtk_combo_box_insert_text((GtkComboBox *)selector4, CAR_LP100KM_INDEX, CAR_LP100KM_TEXT);
+   gtk_combo_box_insert_text((GtkComboBox *)selector4, CAR_LP10KM_INDEX, CAR_LP10KM_TEXT);
+   gtk_combo_box_insert_text((GtkComboBox *)selector4, CAR_MPL_INDEX, CAR_MPL_TEXT);
+   gtk_combo_box_insert_text((GtkComboBox *)selector4, CAR_KMPL_INDEX, CAR_KMPL_TEXT);
+   gtk_table_attach(GTK_TABLE(tablel), selector4, 1, 4, 2, 3, GTK_FILL, GTK_FILL, 2, 2);
+   gtk_signal_connect(GTK_OBJECT(selector4),
+                      "changed",
+                      GTK_SIGNAL_FUNC(car_units_entry_selected_clicked_cb),
+                      selector4);
+
    label6 = gtk_label_new("Cost Entries");
-   gtk_table_attach(GTK_TABLE(tablel), label6, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 2, 2);
+   gtk_table_attach(GTK_TABLE(tablel), label6, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 2, 2);
 
    selector3 = gtk_combo_box_new_text();
    gtk_combo_box_insert_text((GtkComboBox *)selector3, 0, "Fillup");
    gtk_combo_box_insert_text((GtkComboBox *)selector3, 1, "Per Unit");
-   gtk_table_attach(GTK_TABLE(tablel), selector3, 1, 3, 2, 3, GTK_FILL, GTK_FILL, 2, 2);
+   gtk_table_attach(GTK_TABLE(tablel), selector3, 1, 3, 3, 4, GTK_FILL, GTK_FILL, 2, 2);
    gtk_signal_connect(GTK_OBJECT(selector3),
                      "changed",
                      GTK_SIGNAL_FUNC(car_cost_units_clicked_cb),
                      selector3);
 
    label3 = gtk_label_new("Lifetime MPG");
-   gtk_table_attach(GTK_TABLE(tablel), label3, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 2, 2);
+   gtk_table_attach(GTK_TABLE(tablel), label3, 0, 1, 4, 5, GTK_FILL, GTK_FILL, 2, 2);
 
    lifeMPG = gtk_entry_new_with_max_length(20);
    gtk_editable_set_editable(GTK_EDITABLE(lifeMPG), FALSE);
-   gtk_table_attach(GTK_TABLE(tablel), lifeMPG, 1, 3, 3, 4, GTK_FILL, GTK_FILL, 2, 2);
+   gtk_table_attach(GTK_TABLE(tablel), lifeMPG, 1, 3, 4, 5, GTK_FILL, GTK_FILL, 2, 2);
 
    label4 = gtk_label_new("Last MPG");
-   gtk_table_attach(GTK_TABLE(tablel), label4, 0, 1, 4, 5, GTK_FILL, GTK_FILL, 2, 2);
+   gtk_table_attach(GTK_TABLE(tablel), label4, 0, 1, 5, 6, GTK_FILL, GTK_FILL, 2, 2);
 
    lastMPG = gtk_entry_new_with_max_length(20);
    gtk_editable_set_editable(GTK_EDITABLE(lastMPG), FALSE);
-   gtk_table_attach(GTK_TABLE(tablel), lastMPG, 1, 3, 4, 5, GTK_FILL, GTK_FILL, 2, 2);
+   gtk_table_attach(GTK_TABLE(tablel), lastMPG, 1, 3, 5, 6, GTK_FILL, GTK_FILL, 2, 2);
 
    label5 = gtk_label_new("Miles / Month");
-   gtk_table_attach(GTK_TABLE(tablel), label5, 0, 1, 5, 6, GTK_FILL, GTK_FILL, 2, 2);
+   gtk_table_attach(GTK_TABLE(tablel), label5, 0, 1, 6, 7, GTK_FILL, GTK_FILL, 2, 2);
 
    milesMonth = gtk_entry_new_with_max_length(20);
    gtk_editable_set_editable(GTK_EDITABLE(milesMonth), FALSE);
-   gtk_table_attach(GTK_TABLE(tablel), milesMonth, 1, 3, 5, 6, GTK_FILL, GTK_FILL, 2, 2);
+   gtk_table_attach(GTK_TABLE(tablel), milesMonth, 1, 3, 6, 7, GTK_FILL, GTK_FILL, 2, 2);
 
    text_buf = gtk_text_buffer_new(NULL);
    gtk_text_buffer_insert_at_cursor(text_buf, "No Alerts for this Car\n\n", 24);
@@ -220,7 +254,7 @@ void car_tab_layout(GtkWidget *window)
    gtk_container_add(GTK_CONTAINER(sw), GTK_WIDGET(text_box));
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-   gtk_table_attach(GTK_TABLE(tablel), sw, 0, 5, 6, 9, GTK_FILL, GTK_FILL, 2, 2);
+   gtk_table_attach(GTK_TABLE(tablel), sw, 0, 5, 7, 9, GTK_FILL, GTK_FILL, 2, 2);
 }
 
 void
@@ -263,6 +297,12 @@ car_tab_update_units(int units)
    gtk_combo_box_set_active(GTK_COMBO_BOX(selector2), units);
 }
 
+void
+car_tab_update_units_entry(int units)
+{
+   gtk_combo_box_set_active(GTK_COMBO_BOX(selector4), units);
+}
+
 void
 car_tab_update_cost_units(int units)
 {
@@ -429,6 +469,7 @@ car_delete_clicked_cb(GtkWidget* w, gpointer data)
       free(car);
 }
 
+#if GTK_CHECK_VERSION(2,10,0)
 void
 car_export_clicked_cb(GtkWidget* w, gpointer data)
 {
@@ -441,6 +482,9 @@ car_export_clicked_cb(GtkWidget* w, gpointer data)
     };
    GtkWidget *file_label;
    GtkWidget *csv_but, *html_but;
+#ifdef MAEMO
+   GtkWidget *email_label;
+#endif
    int i;
 
    if (running_export == 1)
@@ -469,12 +513,12 @@ car_export_clicked_cb(GtkWidget* w, gpointer data)
        "This allows one to export data from the currently selected car\n\
 to either CVS format or HTML format. The data to be exported will be\n\
 the currently selected data type on the data tab.");
-   page[1].widget = gtk_table_new(5,2,FALSE);
+   page[1].widget = gtk_table_new(5,3,FALSE);
    page[2].widget = gtk_label_new("Please click Apply. If you selected CSV it will write\n\
 out the data to the specified file. If you specified HTML\n\
 then the html file will be loaded into a browser viewer");
 
-   file_label = gtk_button_new_with_label("Select file location and name");
+   file_label = gtk_button_new_with_label("Select file");
    gtk_signal_connect(GTK_OBJECT(file_label),
                      "clicked",
                      GTK_SIGNAL_FUNC(car_export_select_file),
@@ -482,7 +526,7 @@ then the html file will be loaded into a browser viewer");
    gtk_table_attach(GTK_TABLE(page[1].widget), file_label, 0, 1, 0, 1, GTK_FILL, GTK_SHRINK, 2, 2);
 
    export_file_text = gtk_entry_new_with_max_length(200);
-   gtk_table_attach(GTK_TABLE(page[1].widget), export_file_text, 1, 2, 0, 1, GTK_FILL, GTK_SHRINK, 2, 2);
+   gtk_table_attach(GTK_TABLE(page[1].widget), export_file_text, 1, 3, 0, 1, GTK_FILL, GTK_SHRINK, 2, 2);
 
    csv_but = gtk_radio_button_new_with_label(NULL, "CSV Format");
    gtk_table_attach(GTK_TABLE(page[1].widget), csv_but, 0, 1, 1, 2, GTK_FILL, GTK_SHRINK, 2,2);
@@ -491,6 +535,14 @@ then the html file will be loaded into a browser viewer");
    gtk_table_attach(GTK_TABLE(page[1].widget), html_but, 1, 2, 1, 2, GTK_FILL, GTK_SHRINK,2, 2);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(csv_but), TRUE);
 
+#ifdef MAEMO
+   email_label = gtk_label_new("Email file");
+   gtk_table_attach(GTK_TABLE(page[1].widget), email_label, 0, 1, 2, 3, GTK_FILL, GTK_SHRINK, 2, 2);
+
+   email_but = gtk_check_button_new();
+   gtk_table_attach(GTK_TABLE(page[1].widget), email_but, 1, 2, 2, 3, GTK_FILL, GTK_SHRINK, 2, 2);
+#endif
+
    assist_export = gtk_assistant_new();
    for (i = 0; i < sizeof(page) / sizeof(PageInfo); i++)
    {
@@ -943,16 +995,18 @@ car_tab_assist_export_apply(GtkWidget* w, gpointer data)
 
    car = gtk_combo_box_get_active_text(GTK_COMBO_BOX(car_selector));
 
-   units = returnUnits();
-   if (units == 0)
+   units = returnEntryUnits(car);
+   if (units == CAR_MPG_US_INDEX)
       units_str = "MPG";
-   else if (units == 1)
+	else if (units == CAR_MPG_IMP_INDEX)
+      units_str = "MPG";
+   else if (units == CAR_LP100KM_INDEX)
       units_str = "L/100km";
-   else if (units == 2)
+   else if (units == CAR_LP10KM_INDEX)
       units_str = "L/10km";
-   else if (units == 3)
+   else if (units == CAR_MPL_INDEX)
       units_str = "Miles/L";
-   else if (units == 4)
+   else if (units == CAR_KMPL_INDEX)
       units_str = "km/L";
    else
       units_str = "UNKNOWN";
@@ -1022,7 +1076,7 @@ car_tab_assist_export_apply(GtkWidget* w, gpointer data)
          }
          else if (data_selected == DATA_DISPLAY_ACTIVITY)
          {
-            sprintf(buf, "Date,Odometer,Activity,Comment\n");
+            sprintf(buf, "Date,Odometer,Activity,Comment,Cost\n");
             fwrite(buf, strlen(buf), 1, fd);
             list_store = data_tab_get_store();
             valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL(list_store), &iter);
@@ -1033,9 +1087,10 @@ car_tab_assist_export_apply(GtkWidget* w, gpointer data)
                  DATA_ODO_ROW,  &odo_str,
                  DATA_FUEL_ROW, &fuel_str,
                  DATA_MPG_ROW,  &mpg_str,
+                 DATA_COST_ROW, &cost_str,
                  -1);
-               sprintf(buf, "%s,%s,%s,%s\n",
-                  date_str, odo_str, fuel_str, mpg_str);
+               sprintf(buf, "%s,%s,%s,%s,%s\n",
+                  date_str, odo_str, fuel_str, mpg_str,cost_str);
                fwrite(buf, strlen(buf), 1, fd);
                if (date_str)
                   g_free(date_str);
@@ -1045,6 +1100,8 @@ car_tab_assist_export_apply(GtkWidget* w, gpointer data)
                   g_free(fuel_str);
                if (mpg_str)
                   g_free(mpg_str);
+               if (cost_str)
+                  g_free(cost_str);
                valid = gtk_tree_model_iter_next (GTK_TREE_MODEL(list_store), &iter);
             }
          }
@@ -1062,7 +1119,7 @@ car_tab_assist_export_apply(GtkWidget* w, gpointer data)
                  DATA_FUEL_ROW, &fuel_str,
                  DATA_MPG_ROW,  &mpg_str,
                  -1);
-               sprintf(buf, "<br>%s %s %s %s\n",
+               sprintf(buf, "%s %s %s %s\n",
                   date_str, odo_str, fuel_str, mpg_str);
                fwrite(buf, strlen(buf), 1, fd);
                if (date_str)
@@ -1178,7 +1235,7 @@ car_tab_assist_export_apply(GtkWidget* w, gpointer data)
             sprintf(buf, "<h2>Car Activity Data for: %s</h2>\n", car);
             fwrite(buf, strlen(buf), 1, fd);
 #if 0
-            sprintf(buf, "<h2>Date  Odometer  Activity  Comment</h2>\n");
+            sprintf(buf, "<h2>Date  Odometer  Activity  Comment  Cost</h2>\n");
             fwrite(buf, strlen(buf), 1, fd);
 #endif
             list_store = data_tab_get_store();
@@ -1187,7 +1244,7 @@ car_tab_assist_export_apply(GtkWidget* w, gpointer data)
             fwrite(buf, strlen(buf), 1, fd);
             sprintf(buf, "document.write(\"<XMP>\")\n");
             fwrite(buf, strlen(buf), 1, fd);
-            sprintf(buf, "document.write(\"Date    \\tOdometer  \\tActivity                        \\tComment\\r\")\n");
+            sprintf(buf, "document.write(\"Date    \\tOdometer  \\tActivity                 \\tComment    \\tCost    \\r\")\n");
             fwrite(buf, strlen(buf), 1, fd);
             while (valid)
             {
@@ -1196,9 +1253,10 @@ car_tab_assist_export_apply(GtkWidget* w, gpointer data)
                  DATA_ODO_ROW,  &odo_str,
                  DATA_FUEL_ROW, &fuel_str,
                  DATA_MPG_ROW,  &mpg_str,
+                 DATA_COST_ROW, &cost_str,
                  -1);
-               sprintf(buf, "document.write(\"%s\\t%8s\\t%25s\\t%s\\r\")\n",
-                  date_str, odo_str, fuel_str, mpg_str);
+               sprintf(buf, "document.write(\"%s\\t%8s\\t%25s\\t%s\\t%10s\\r\")\n",
+                  date_str, odo_str, fuel_str, mpg_str, cost_str);
                fwrite(buf, strlen(buf), 1, fd);
                if (date_str)
                   g_free(date_str);
@@ -1208,6 +1266,8 @@ car_tab_assist_export_apply(GtkWidget* w, gpointer data)
                   g_free(fuel_str);
                if (mpg_str)
                   g_free(mpg_str);
+               if (cost_str)
+                  g_free(cost_str);
                valid = gtk_tree_model_iter_next (GTK_TREE_MODEL(list_store), &iter);
             }
             sprintf(buf, "document.write(\"</XMP>\")\n");
@@ -1323,6 +1383,37 @@ car_tab_assist_export_apply(GtkWidget* w, gpointer data)
            DBUS_TYPE_STRING, buf, DBUS_TYPE_BOOLEAN, FALSE, DBUS_TYPE_INVALID);
 #endif
       }
+
+#ifdef MAEMO
+      /* EMAIL the file if the user requested it */
+      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(email_but)))
+      {
+         gboolean result = TRUE;
+         GSList *list = NULL;
+         sprintf(buf, "Attached to this email is the output from the Car Mileage Calculator for car %s", car);
+         /* Attach the saved file (and not the one currently on screen). If the file
+         * has not yet been saved, nothing will be attached */
+         list = g_slist_append(list, file_text);
+         result = libmodest_dbus_client_compose_mail(mileage_get_osso(), /*osso_context_t*/
+            NULL, /*to*/
+            NULL, /*cc*/
+            NULL, /*bcc*/
+            "Car Mileage Calculator - Export", /*subject*/
+            buf, /*body*/
+            list /*attachments*/);
+         g_slist_free(list);
+         if (result == FALSE)
+         {
+            dialog = gtk_message_dialog_new (NULL,
+               GTK_DIALOG_DESTROY_WITH_PARENT,
+               GTK_MESSAGE_ERROR,
+               GTK_BUTTONS_CLOSE,
+               "Error, Can't email file");
+            gtk_dialog_run (GTK_DIALOG (dialog));
+            gtk_widget_destroy (dialog);
+         }
+      }
+#endif
    }
 
    gtk_widget_destroy(GTK_WIDGET(assist_export));
@@ -1359,7 +1450,11 @@ car_tab_assist_apply(GtkWidget* w, gpointer data)
       for (i = 1; i < lines; i++)
       {
          gtk_text_buffer_get_iter_at_line(GTK_TEXT_BUFFER(assist_text_buf), &start, i);
-         gtk_text_buffer_get_iter_at_line(GTK_TEXT_BUFFER(assist_text_buf), &end, i + 1);
+         /* IF last line in file then get an end iterator */
+         if (lines == (i + 1))
+            gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(assist_text_buf), &end);
+         else
+            gtk_text_buffer_get_iter_at_line(GTK_TEXT_BUFFER(assist_text_buf), &end, i + 1);
          total_str = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(assist_text_buf), &start, &end, FALSE);
 
          /* Make sure this is not a blank line */
@@ -1378,6 +1473,7 @@ car_tab_assist_apply(GtkWidget* w, gpointer data)
    }
 
    common_tab_updateDisplayTab(car);
+   graph_tab_load_data(car);
 
    if (car)
       free(car);
@@ -1385,6 +1481,7 @@ car_tab_assist_apply(GtkWidget* w, gpointer data)
    running_import = 0;
    gtk_widget_destroy(GTK_WIDGET(assist_import));
 }
+#endif
 
 void
 car_units_selected_clicked_cb(GtkWidget* w, gpointer data)
@@ -1408,6 +1505,36 @@ car_units_selected_clicked_cb(GtkWidget* w, gpointer data)
          car[strlen(car) - 1] = '\0';
       }
       /* Update all entries after delete */
+      common_tab_load_data(car);
+      common_tab_updateDisplayTab(car);
+   }
+   if (car)
+      g_free(car);
+}
+
+void
+car_units_entry_selected_clicked_cb(GtkWidget* w, gpointer data)
+{
+   int  value;
+   char *car;
+
+#ifdef DEBUG
+   fprintf(stdout, "Entry Units Selected\n");
+#endif
+
+   car = gtk_combo_box_get_active_text(GTK_COMBO_BOX(car_selector));
+   if (car != NULL)
+   {
+      if (car[strlen(car) - 1] == '\n')
+      {
+         car[strlen(car) - 1] = '\0';
+      }
+      value = gtk_combo_box_get_active(GTK_COMBO_BOX(data));
+      if (value != -1)
+         addEntryUnits(car, value);
+
+      /* Update all entries after delete */
+      common_tab_load_data(car);
       common_tab_updateDisplayTab(car);
    }
    if (car)
@@ -1446,3 +1573,61 @@ car_tab_get_text_buffer()
    return (GTK_WIDGET(text_buf));
 }
 
+void
+car_backup_clicked_cb(GtkWidget* w, gpointer data)
+{
+   GDate  *date = NULL;
+   GDateYear  year;
+   GDateMonth month;
+   GDateDay   day;
+   char       str[512];
+
+   str[0] = '0';
+   date = g_date_new();
+   if (date) 
+   {
+      /* IF GLIB 2.10 or later */
+      if (GLIB_CHECK_VERSION(2,10,0))
+         g_date_set_time_t(date, time(NULL));
+      else
+         g_date_set_time(date, (GTime)time(NULL));
+      year = g_date_get_year(date);
+      month= g_date_get_month(date);
+      day  = g_date_get_day(date);
+      g_date_free(date);
+      sprintf(str, "%d/%d/%d", (int)year, (int)month, (int)day);
+   }
+
+#ifdef MAEMO
+   char       buf[1024];
+   GtkWidget       *dialog;
+   gboolean result = TRUE;
+   GSList *list = returnCarFiles();
+   sprintf(buf, "Attached to this email are the data files from the Car Mileage Calculator generated on %s.\n\
+To restore place these files into the mileage directory under the MyDocs directory. Be careful not to \
+write over any files that are already there.", str);
+   /* Attach the saved file (and not the one currently on screen). If the file
+   * has not yet been saved, nothing will be attached */
+   result = libmodest_dbus_client_compose_mail(mileage_get_osso(), /*osso_context_t*/
+      NULL, /*to*/
+      NULL, /*cc*/
+      NULL, /*bcc*/
+      "Car Mileage Calculator - Backup", /*subject*/
+      buf, /*body*/
+      list /*attachments*/);
+
+   g_slist_foreach(list, common_delete_data, NULL);
+   g_slist_free(list);
+   if (result == FALSE)
+   {
+      dialog = gtk_message_dialog_new (NULL,
+         GTK_DIALOG_DESTROY_WITH_PARENT,
+         GTK_MESSAGE_ERROR,
+         GTK_BUTTONS_CLOSE,
+         "Error, Can't email file");
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+   }
+#endif
+}
+
