@@ -45,6 +45,8 @@ static void common_tab_select_car_sub(char *, GtkWidget *, int);
 static void common_tab_updateActivityData(char *);
 static void common_tab_updateAlertData(char *);
 static void common_tab_updateTripData(char *);
+static void convertDist(double, double*, int, int);
+
 
 /* Loads a car into the Car selection menu. NULL means clear all entries */
 void
@@ -199,6 +201,7 @@ common_tab_updateDisplayTab(char *car)
 {
    GtkListStore *store;
    int           selector;
+	int           entry_units;
 
    /* Clear the list */
    store = data_tab_get_store();
@@ -231,6 +234,20 @@ common_tab_updateDisplayTab(char *car)
       common_tab_updateGasData(FALSE, car);
       common_tab_updateTripData(car);
    }
+
+   /* Update Entry Units */
+   entry_units = returnEntryUnits(car);
+   /* If an entry units is stored in the data file then get it */
+   if (entry_units != -1)
+      car_tab_update_units_entry(entry_units);
+   else
+   {
+      /* If no entry units in file then use Display Units and save to file */
+      entry_units = units;
+      car_tab_update_units_entry(entry_units);
+      if (car != NULL)
+         addEntryUnits(car, entry_units);
+   }
 }
 
 void
@@ -238,6 +255,7 @@ common_tab_updateActivityData(char *car)
 {
    GSList  *lst;
    GSList *lst_iter;
+   GSList *lst_first;
    GtkTreeIter   iter;
    char buf[1024];
    char *saveptr;
@@ -245,16 +263,18 @@ common_tab_updateActivityData(char *car)
    char *odometer;
    char *activity;
    char *comment;
+   char *cost;
    char *length;
    int  len;
    GtkListStore *store;
 
-   data_tab_update_columns("Date", "Odometer", "Activity", "Comment", "", "", "");
+   data_tab_update_columns("Date", "Odometer", "Activity", "Comment", "Cost", "", "");
 
    lst = returnActivityEntries(car);
 
    /* sort the list in date order */
    lst_iter = g_slist_sort(lst, common_sort_date);
+   lst_first = lst_iter;
 
    store = data_tab_get_store();
 
@@ -271,17 +291,28 @@ common_tab_updateActivityData(char *car)
       length = (char *)strtok_r(NULL, " ", &saveptr);
       len = atoi(length);
       comment = saveptr;
+      saveptr[len] = '\0';
+      saveptr += (len + 1);
+      length = (char *)strtok_r(NULL, " ", &saveptr);
+      /* Data file may not contain a cost entry */
+      if (length != NULL)
+      {
+         len = atoi(length);
+         cost = saveptr;
+      }
+      else
+         cost = "";
 
       gtk_list_store_append(store, &iter);
-      gtk_list_store_set(store, &iter, 0, date, 1, odometer, 2, activity, 3, comment, 4, "", -1);
+      gtk_list_store_set(store, &iter, 0, date, 1, odometer, 2, activity, 3, comment, 4, cost, -1);
 
       free(lst_iter->data);
       lst_iter->data = NULL;
       lst_iter = g_slist_next(lst_iter);
    }
 
-   g_slist_foreach(lst_iter, common_delete_data, NULL);
-   g_slist_free(lst_iter);
+   g_slist_foreach(lst_first, common_delete_data, NULL);
+   g_slist_free(lst_first);
 }
 
 void
@@ -289,6 +320,7 @@ common_tab_updateAlertData(char *car)
 {
    GSList  *lst;
    GSList *lst_iter;
+   GSList *lst_first;
    GtkTreeIter   iter;
    char buf[1024];
    char *saveptr;
@@ -306,7 +338,8 @@ common_tab_updateAlertData(char *car)
 
    /* sort the list in date order */
    lst_iter = g_slist_sort(lst, common_sort_date);
-
+   lst_first = lst_iter;
+   
    store = data_tab_get_store();
 
    while (lst_iter != NULL)
@@ -331,8 +364,8 @@ common_tab_updateAlertData(char *car)
       lst_iter = g_slist_next(lst_iter);
    }
 
-   g_slist_foreach(lst_iter, common_delete_data, NULL);
-   g_slist_free(lst_iter);
+   g_slist_foreach(lst_first, common_delete_data, NULL);
+   g_slist_free(lst_first);
 }
 
 void
@@ -340,6 +373,7 @@ common_tab_updateTripData(char *car)
 {
    GSList  *lst;
    GSList *lst_iter;
+   GSList *lst_first;
    GtkTreeIter   iter;
    char buf[1024];
    char *saveptr;
@@ -361,6 +395,7 @@ common_tab_updateTripData(char *car)
 
    /* sort the list in date order */
    lst_iter = g_slist_sort(lst, common_sort_date);
+   lst_first = lst_iter;
 
    store = data_tab_get_store();
 
@@ -399,8 +434,8 @@ common_tab_updateTripData(char *car)
       lst_iter = g_slist_next(lst_iter);
    }
 
-   g_slist_foreach(lst_iter, common_delete_data, NULL);
-   g_slist_free(lst_iter);
+   g_slist_foreach(lst_first, common_delete_data, NULL);
+   g_slist_free(lst_first);
 }
 
 void
@@ -417,6 +452,7 @@ common_tab_updateGasData(int update_tab, char *car)
    char     *fuel_label;
    GSList *lst;
    GSList *lst_iter;
+   GSList *lst_first;
    GtkTreeIter   iter;
    char *date;   
    char *odometer;
@@ -445,6 +481,7 @@ common_tab_updateGasData(int update_tab, char *car)
    double last_cost;
    GtkListStore *store;
    double   miles_month = 0.0;
+	int    entry_units;
 
    first_date[0] = '\0';
    last_date[0] = '\0';
@@ -453,63 +490,94 @@ common_tab_updateGasData(int update_tab, char *car)
 
    if (update_tab)
    {
+      entry_units = returnEntryUnits(car);
+
       date_string = "Date";
       odo_string = "Odometer";
       cost_string = "Cost";
 
+      /* Set correct labels for gas data */
+      switch (entry_units)
+      {
+         /* MPG */
+         case CAR_MPG_US_INDEX:
+         case CAR_MPG_IMP_INDEX:
+            fuel_label = "Enter Gallons";
+            fuel_string = "Gallons";
+            miles_month_label = "Miles / Month:";
+         break;
+
+         /* L/100km */
+         case CAR_LP100KM_INDEX:
+         /* L/10km */
+         case CAR_LP10KM_INDEX:
+         /* km / L */
+         case CAR_KMPL_INDEX:
+            fuel_label = "Enter Liters";
+            fuel_string = "Liters";
+            miles_month_label = "Km / Month:";
+         break;
+
+         /* Miles / L */
+         case CAR_MPL_INDEX:
+            fuel_label = "Enter Liters";
+            fuel_string = "Liters";
+            miles_month_label = "Miles / Month:";
+         break;
+
+         default:
+            fuel_label = "";
+            fuel_string = "";
+            miles_month_label = "";
+         break;
+      }
+
       /* Set correct labels for gas data */
       switch (units)
       {
          /* MPG */
-         case 0:
+         case CAR_MPG_US_INDEX:
+         case CAR_MPG_IMP_INDEX:
             lifetime_label = "Lifetime MPG:";
             last_label = "Last MPG:";
-            miles_month_label = "Miles / Month:";
-            fuel_label = "Enter Gallons";
-            fuel_string = "Gallons";
             mpg_string = "MPG";
          break;
 
          /* L/100km */
-         case 1:
+         case CAR_LP100KM_INDEX:
             lifetime_label = "Lifetime L/100km:";
             last_label = "Last L/100km:";
-            miles_month_label = "Kilometers / Month:";
-            fuel_label = "Enter Liters";
-            fuel_string = "Liters";
             mpg_string = "L/100km";
          break;
 
          /* L/10km */
-         case 2:
+         case CAR_LP10KM_INDEX:
             lifetime_label = "Lifetime L/10km:";
             last_label = "Last L/10km:";
-            miles_month_label = "Kilometers / Month:";
-            fuel_label = "Enter Liters";
-            fuel_string = "Liters";
             mpg_string = "L/10km";
          break;
 
          /* Miles / L */
-         case 3:
+         case CAR_MPL_INDEX:
             lifetime_label = "Lifetime Miles/L:";
             last_label = "Last Miles/L:";
-            miles_month_label = "Miles / Month:";
-            fuel_label = "Enter Liters";
-            fuel_string = "Liters";
             mpg_string = "M/L";
          break;
 
          /* km / L */
-         case 4:
+         case CAR_KMPL_INDEX:
             lifetime_label = "Lifetime km/L:";
             last_label = "Last km/L:";
-            miles_month_label = "Kilometers / Month:";
-            fuel_label = "Enter Liters";
-            fuel_string = "Liters";
             mpg_string = "km/L";
          break;
+
+         default:
+            lifetime_label = "";
+            last_label = "";
+            mpg_string = "";
+         break;
       }
+
       data_tab_update_columns(date_string, odo_string, fuel_string, mpg_string, cost_string, "", "");
       car_tab_update_text_labels(lifetime_label, last_label, miles_month_label);
       entry_tab_update_text_label(fuel_label);
@@ -521,6 +589,7 @@ common_tab_updateGasData(int update_tab, char *car)
 
       /* sort the list in date order */
       lst_iter = g_slist_sort(lst, common_sort_date);
+      lst_first = lst_iter;
 
       store = data_tab_get_store();
 
@@ -562,22 +631,28 @@ common_tab_updateGasData(int update_tab, char *car)
                total_odo += (last_odo - prev_odo);
                total_fuel += last_fuel;
 
-               /* MPG */
-               if (units == 0)
+               /* MPG US */
+               if (entry_units == CAR_MPG_US_INDEX)
+                  mileage_d = (double)(last_odo - prev_odo) / last_fuel;
+               /* MPG IMP */
+               else if (entry_units == CAR_MPG_IMP_INDEX)
                   mileage_d = (double)(last_odo - prev_odo) / last_fuel;
                /* L/100km */
-               else if (units == 1)
+               else if (entry_units == CAR_LP100KM_INDEX)
                   mileage_d = 100.0 * last_fuel / (double)(last_odo - prev_odo);
                /* L/10km */
-               else if (units == 2)
+               else if (entry_units == CAR_LP10KM_INDEX)
                   mileage_d = 10.0 * last_fuel / (double)(last_odo - prev_odo);
                /* Miles / L */
-               else if (units == 3)
+               else if (entry_units == CAR_MPL_INDEX)
                   mileage_d = (double)(last_odo - prev_odo) / last_fuel;
                /* km / L */
-               else if (units == 4)
+               else if (entry_units == CAR_KMPL_INDEX)
                   mileage_d = (double)(last_odo - prev_odo) / last_fuel;
+               else
+                  fprintf(stdout, "Invalid entry units %d\n", entry_units);
 
+               convertMPG(mileage_d, &mileage_d, entry_units, units);
                sprintf(mileage, "%6.2f", mileage_d);
             }
          }
@@ -599,43 +674,50 @@ common_tab_updateGasData(int update_tab, char *car)
             cost_total += (last_cost * last_fuel);
 
       }
-      g_slist_foreach(lst_iter, common_delete_data, NULL);
-      g_slist_free(lst_iter);
+      g_slist_foreach(lst_first, common_delete_data, NULL);
+      g_slist_free(lst_first);
 
       /* If there is data to calculate lifetime MPG */
       if (total_fuel > 0.0)
       {
-         /* MPG */
-         if (units == 0)
+         /* MPG US */
+         if (entry_units == CAR_MPG_US_INDEX)
+         {
+            total_fuel = (double)total_odo / total_fuel;
+            gallons = (double)big_odo / total_fuel;
+         }
+         /* MPG IMP */
+         else if (entry_units == CAR_MPG_IMP_INDEX)
          {
             total_fuel = (double)total_odo / total_fuel;
             gallons = (double)big_odo / total_fuel;
          }
          /* L/100km */
-         else if (units == 1)
+         else if (entry_units == CAR_LP100KM_INDEX)
          {
             total_fuel = 100.0 * total_fuel / (double)total_odo;
             gallons = total_fuel / 100.0 * big_odo;
          }
          /* L/10km */
-         else if (units == 2)
+         else if (entry_units == CAR_LP10KM_INDEX)
          {
             total_fuel = 10.0 * total_fuel / (double)total_odo;
             gallons = total_fuel / 10.0 * big_odo;
          }
          /* Miles / L */
-         else if (units == 3)
+         else if (entry_units == CAR_MPL_INDEX)
          {
             total_fuel = (double)total_odo / total_fuel;
             gallons = (double)big_odo / total_fuel;
          }
          /* km / L */
-         else if (units == 4)
+         else if (entry_units == CAR_KMPL_INDEX)
          {
             total_fuel = (double)total_odo / total_fuel;
             gallons = (double)big_odo / total_fuel;
          }
 
+         convertMPG(total_fuel, &total_fuel, entry_units, units);
          sprintf(lifeMPG, "%6.2f", total_fuel);
       }
 
@@ -649,6 +731,7 @@ common_tab_updateGasData(int update_tab, char *car)
          if (g_date_valid(date1) == TRUE && g_date_valid(date2) == TRUE)
          {
             miles_month = (last_odo - first_odo) * 30.0 / g_date_days_between(date1, date2);
+            /* convertDist(miles_month, &miles_month, entry_units, units); */
             sprintf(miles_month_str, "%7.1f", miles_month);
          }
          g_date_free(date1);
@@ -735,6 +818,7 @@ common_tab_updateAlertDisplay(char *car)
 {
    GSList *lst;
    GSList *lst_iter;
+   GSList *lst_first;
    GtkWidget  *text_buf;
    GtkTextIter  start;
    GtkTextIter  end;
@@ -761,6 +845,7 @@ common_tab_updateAlertDisplay(char *car)
    {
       gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(text_buf), "Alerts for this car are:\n", 25);
       lst_iter = lst;
+      lst_first = lst_iter;
       while (lst_iter != NULL)
       {
          odo = strtok_r(lst_iter->data, " ", &saveptr);
@@ -781,6 +866,161 @@ common_tab_updateAlertDisplay(char *car)
          lst_iter->data = NULL;
          lst_iter = g_slist_next(lst_iter);
       }
+      g_slist_foreach(lst_first, common_delete_data, NULL);
+      g_slist_free(lst_first);
    }
 }
 
+void
+convertMPG(
+   double in_value,
+   double* out_value,
+   int    in_units,
+   int    out_units)
+{
+   switch (out_units)
+   {
+      case CAR_MPG_US_INDEX:
+         if (in_units == CAR_MPG_US_INDEX)
+            *out_value = in_value;
+         if (in_units == CAR_MPG_IMP_INDEX)
+            *out_value = in_value / 1.20095042;
+         else if (in_units == CAR_LP100KM_INDEX)
+            *out_value = 235.214584 / in_value;
+         else if (in_units == CAR_LP10KM_INDEX)
+            *out_value = 23.5214584 / in_value;
+         else if (in_units == CAR_MPL_INDEX)
+            *out_value = in_value  / 0.26417205;
+         else if (in_units == CAR_KMPL_INDEX)
+            *out_value = in_value / 0.4251437;
+      break;
+
+      case CAR_MPG_IMP_INDEX:
+         if (in_units == CAR_MPG_US_INDEX)
+            *out_value = in_value * 1.20095042;
+         if (in_units == CAR_MPG_IMP_INDEX)
+            *out_value = in_value;
+         else if (in_units == CAR_LP100KM_INDEX)
+            *out_value = 1.20095042 * 235.214584 / in_value;
+         else if (in_units == CAR_LP10KM_INDEX)
+            *out_value = 1.20095042 * 23.5214584 / in_value;
+         else if (in_units == CAR_MPL_INDEX)
+            *out_value = 1.20095042 * in_value  / 0.26417205;
+         else if (in_units == CAR_KMPL_INDEX)
+            *out_value = 1.20095042 * in_value / 0.4251437;
+      break;
+
+      case CAR_LP100KM_INDEX:
+         if (in_units == CAR_MPG_US_INDEX)
+            *out_value = 235.214584 / in_value;
+         if (in_units == CAR_MPG_IMP_INDEX)
+            *out_value = 235.214584 / (in_value / 1.20095042);
+         else if (in_units == CAR_LP100KM_INDEX)
+            *out_value = in_value;
+         else if (in_units == CAR_LP10KM_INDEX)
+            *out_value = 10.0 * in_value;
+         else if (in_units == CAR_MPL_INDEX)
+            *out_value = 62.137119 / in_value;
+         else if (in_units == CAR_KMPL_INDEX)
+            *out_value = 100.0 / in_value;
+      break;
+
+      case CAR_LP10KM_INDEX:
+         if (in_units == CAR_MPG_US_INDEX)
+            *out_value = 23.5214584 / in_value;
+         if (in_units == CAR_MPG_IMP_INDEX)
+            *out_value = 23.5214584 / (in_value / 1.20095042);
+         else if (in_units == CAR_LP100KM_INDEX)
+            *out_value = in_value / 10.0;
+         else if (in_units == CAR_LP10KM_INDEX)
+            *out_value = in_value;
+         else if (in_units == CAR_MPL_INDEX)
+            *out_value = 6.2137119 / in_value;
+         else if (in_units == CAR_KMPL_INDEX)
+            *out_value = 10.0 / in_value;
+      break;
+
+      case CAR_MPL_INDEX:
+         if (in_units == CAR_MPG_US_INDEX)
+            *out_value = in_value / 3.785412;
+         if (in_units == CAR_MPG_IMP_INDEX)
+            *out_value = (in_value / 1.20095042) / 3.785412;
+         else if (in_units == CAR_LP100KM_INDEX)
+            *out_value = 62.137119 / in_value;
+         else if (in_units == CAR_LP10KM_INDEX)
+            *out_value = 6.2137119 / in_value;
+         else if (in_units == CAR_MPL_INDEX)
+            *out_value = in_value;
+         else if (in_units == CAR_KMPL_INDEX)
+            *out_value = in_value / 1.609344;
+      break;
+
+      case CAR_KMPL_INDEX:
+         if (in_units == CAR_MPG_US_INDEX)
+            *out_value = 0.42505281 * in_value;
+         if (in_units == CAR_MPG_IMP_INDEX)
+            *out_value = 1.20095042 / 0.42505281 * in_value;
+         else if (in_units == CAR_LP100KM_INDEX)
+            *out_value = 100.0 / in_value;
+         else if (in_units == CAR_LP10KM_INDEX)
+            *out_value = 10.0 / in_value;
+         else if (in_units == CAR_MPL_INDEX)
+            *out_value = 1.609344 * in_value;
+         else if (in_units == CAR_KMPL_INDEX)
+            *out_value = in_value;
+      break;
+ 
+      default:
+            *out_value = in_value;
+      break;
+   }
+}
+
+void
+convertDist(
+   double in_value,
+   double* out_value,
+   int    in_units,
+   int    out_units)
+{
+   switch (out_units)
+   {
+      case CAR_MPG_US_INDEX:
+      case CAR_MPG_IMP_INDEX:
+      case CAR_MPL_INDEX:
+         if (in_units == CAR_MPG_US_INDEX)
+            *out_value = in_value;
+         if (in_units == CAR_MPG_IMP_INDEX)
+            *out_value = in_value;
+         else if (in_units == CAR_LP100KM_INDEX)
+            *out_value = in_value / 1.609344;
+         else if (in_units == CAR_LP10KM_INDEX)
+            *out_value = in_value / 1.609344;
+         else if (in_units == CAR_MPL_INDEX)
+            *out_value = in_value;
+         else if (in_units == CAR_KMPL_INDEX)
+            *out_value = in_value / 1.609344;
+      break;
+
+      case CAR_LP100KM_INDEX:
+      case CAR_LP10KM_INDEX:
+      case CAR_KMPL_INDEX:
+         if (in_units == CAR_MPG_US_INDEX)
+            *out_value = 1.609344 * in_value;
+         if (in_units == CAR_MPG_IMP_INDEX)
+            *out_value = 1.609344 * in_value;
+         else if (in_units == CAR_LP100KM_INDEX)
+            *out_value = in_value;
+         else if (in_units == CAR_LP10KM_INDEX)
+            *out_value = in_value;
+         else if (in_units == CAR_MPL_INDEX)
+            *out_value = 1.609344 * in_value;
+         else if (in_units == CAR_KMPL_INDEX)
+            *out_value = in_value;
+      break;
+
+      default:
+            *out_value = in_value;
+      break;
+   }
+}
