@@ -26,6 +26,7 @@
 #include <gtk/gtk.h>
 
 #include "common_tab.h"
+#include "car_tab.h"
 #include "graph_tab.h"
 #include "mileagecfg.h"
 #include "mileagefile.h"
@@ -313,34 +314,40 @@ graph_expose_event(
    switch (units)
    {
       /* MPG */
-      case 0:
+      case CAR_MPG_US_INDEX:
+      case CAR_MPG_IMP_INDEX:
          dist_str = "M/Day";
          mile_str = "MPG";
       break;
 
       /* L/100km */
-      case 1:
+      case CAR_LP100KM_INDEX:
          dist_str = "Km/Day";
          mile_str = "L/100km";
       break;
 
       /* L/10km */
-      case 2:
+      case CAR_LP10KM_INDEX:
          dist_str = "Km/Day";
          mile_str = "L/10km";
       break;
 
       /* Miles / L */
-      case 3:
+      case CAR_MPL_INDEX:
          dist_str = "M/Day";
          mile_str = "M/L";
       break;
 
       /* km / L */
-      case 4:
+      case CAR_KMPL_INDEX:
          dist_str = "Km/Day";
          mile_str = "Km/L";
       break;
+
+      default:
+         dist_str = "";
+         mile_str = "";
+      break;
    }
 
    /* Based upon graph selection find max scale value */
@@ -680,6 +687,7 @@ graph_mileage_year(
    double   prev_fuel = 0.0;
    double   mpg;
    int      units;
+   int      entry_units;
    int      x_val, y_val;
    int      p_x = -1;
    int      p_y = -1;
@@ -687,10 +695,17 @@ graph_mileage_year(
    int      days_in_month;
    char    *saveptr;
    char    *cost_str;
+   char    *car;
 
+   car = gtk_combo_box_get_active_text(GTK_COMBO_BOX(car_selector));
+   if (car[strlen(car) - 1] == '\n')
+   {
+      car[strlen(car) - 1] = '\0';
+   }
 
    data_item = g_string_new(NULL);
    units = returnUnits();
+   entry_units = returnEntryUnits(car);
 
    iter = sort_list;
    while (iter != NULL)
@@ -725,18 +740,21 @@ graph_mileage_year(
          }
          else
          {
-            if (units == 0)
+            if (entry_units == CAR_MPG_US_INDEX)
                mpg = (odometer - prev_odo) / fuel;
-            else if (units == 1)
+            if (entry_units == CAR_MPG_IMP_INDEX)
+               mpg = (odometer - prev_odo) / fuel;
+            else if (entry_units == CAR_LP100KM_INDEX)
                mpg = 100.0 * fuel / (odometer - prev_odo);
-            else if (units == 2)
+            else if (entry_units == CAR_LP10KM_INDEX)
                mpg = 10.0 * fuel / (odometer - prev_odo);
-            else if (units == 3)
+            else if (entry_units == CAR_MPL_INDEX)
                mpg = (odometer - prev_odo) / fuel;
-            else if (units == 4)
+            else if (entry_units == CAR_KMPL_INDEX)
                mpg = (odometer - prev_odo) / fuel;
             else
                mpg = (odometer - prev_odo) / fuel;
+            convertMPG(mpg, &mpg, entry_units, units);
 
             y_val = y_max - (int)(mpg * ((y_max - Y_OFFSET) / (double)max_display));
             x_val = ((int)g_date_get_month(&gdate) - 1) * X_DELTA + X_OFFSET;
@@ -744,14 +762,8 @@ graph_mileage_year(
 
             if (p_x != -1)
             {
-               if (x_val > X_MAX_NUM)
-                  x_val = X_MAX_NUM;
-               if (x_val < 0)
-                  x_val = 0;
-               if (y_val > Y_MAX_NUM)
-                  y_val = Y_MAX_NUM;
-               if (y_val < 0)
-                  y_val = 0;
+               x_val = CLAMP(x_val, 0, X_MAX_NUM);
+               y_val = CLAMP(y_val, 0, Y_MAX_NUM);
 
                gdk_draw_line(area->window, area->style->fg_gc[GTK_WIDGET_STATE(area)],
                   p_x, p_y, x_val, y_val);
@@ -770,6 +782,8 @@ graph_mileage_year(
       iter = g_slist_next(iter);
    }
 
+   if (car)
+      free(car);
    g_string_free(data_item, TRUE);
 }
 
@@ -845,14 +859,8 @@ graph_distance_year(
 
             if (p_x != -1)
             {
-               if (x_val > X_MAX_NUM)
-                  x_val = X_MAX_NUM;
-               if (x_val < 0)
-                  x_val = 0;
-               if (y_val > Y_MAX_NUM)
-                  y_val = Y_MAX_NUM;
-               if (y_val < 0)
-                  y_val = 0;
+               x_val = CLAMP(x_val, 0, X_MAX_NUM);
+               y_val = CLAMP(y_val, 0, Y_MAX_NUM);
 
                gdk_draw_line(area->window, area->style->fg_gc[GTK_WIDGET_STATE(area)],
                   p_x, p_y, x_val, y_val);
@@ -958,14 +966,8 @@ graph_cost_year(
 
                if (p_x != -1)
                {
-                  if (x_val > X_MAX_NUM)
-                     x_val = X_MAX_NUM;
-                  if (x_val < 0)
-                     x_val = 0;
-                  if (y_val > Y_MAX_NUM)
-                     y_val = Y_MAX_NUM;
-                  if (y_val < 0)
-                     y_val = 0;
+                  x_val = CLAMP(x_val, 0, X_MAX_NUM);
+                  y_val = CLAMP(y_val, 0, Y_MAX_NUM);
 
                   gdk_draw_line(area->window, area->style->fg_gc[GTK_WIDGET_STATE(area)],
                      p_x, p_y, x_val, y_val);
@@ -1074,9 +1076,19 @@ graph_get_max_mileage()
    double   mpg;
    double   prev_fuel;
    int      units;
+   int      entry_units;
+   char    *car;
+
+
+   car = gtk_combo_box_get_active_text(GTK_COMBO_BOX(car_selector));
+   if (car[strlen(car) - 1] == '\n')
+   {
+      car[strlen(car) - 1] = '\0';
+   }
 
    data_item = g_string_new(NULL);
    units = returnUnits();
+   entry_units = returnEntryUnits(car);
 
    iter = sort_list;
    while (iter != NULL)
@@ -1107,18 +1119,21 @@ graph_get_max_mileage()
       }
       else
       {
-         if (units == 0)
+         if (entry_units == CAR_MPG_US_INDEX)
             mpg = (odometer - prev_odo) / fuel;
-         else if (units == 1)
+         if (entry_units == CAR_MPG_IMP_INDEX)
+            mpg = (odometer - prev_odo) / fuel;
+         else if (entry_units == CAR_LP100KM_INDEX)
             mpg = 100.0 * fuel / (odometer - prev_odo);
-         else if (units == 2)
+         else if (entry_units == CAR_LP10KM_INDEX)
             mpg = 10.0 * fuel / (odometer - prev_odo);
-         else if (units == 3)
+         else if (entry_units == CAR_MPL_INDEX)
             mpg = (odometer - prev_odo) / fuel;
-         else if (units == 4)
+         else if (entry_units == CAR_KMPL_INDEX)
             mpg = (odometer - prev_odo) / fuel;
          else
             mpg = (odometer - prev_odo) / fuel;
+         convertMPG(mpg, &mpg, entry_units, units);
 
          if (mpg > max_mpg)
             max_mpg = mpg;
@@ -1130,6 +1145,9 @@ graph_get_max_mileage()
    }
    g_string_free(data_item, TRUE);
 
+   if (car)
+      free(car);
+
 #ifdef DEBUG
       fprintf(stdout, "Max mpg is %f\n", max_mpg);
 #endif
