@@ -20,20 +20,21 @@
  */
 
 
-#ifdef MAEMO
-#ifdef GREGALE
-#include <hildon-widgets/hildon-program.h>
-#else
-#include <hildon/hildon-program.h>
-#include <hildon/hildon-help.h>
-#endif
-#include <libosso.h>
-#endif
 #include <gtk/gtkmain.h>
 #include <gtk/gtkbutton.h>
 #include <gtk/gtktoolbar.h>
 #include <gtk/gtk.h>
 
+#ifdef MAEMO
+#if GTK_CHECK_VERSION(2,10,0)
+#include <hildon/hildon-program.h>
+#include <hildon/hildon-help.h>
+#else
+#include <hildon-widgets/hildon-program.h>
+#endif
+#include <libosso.h>
+#endif
+
 #include <stdlib.h>
 
 #include "car_tab.h"
@@ -44,7 +45,7 @@
 #include "graph_tab.h"
 #include "common_tab.h"
 
-#define MILEAGE_VERSION "1.1"
+#define MILEAGE_VERSION "1.2"
 
 static gint delete_event_cb(GtkWidget* w, GdkEventAny* e, gpointer data);
 static void create_main_menu(GtkWidget *w);
@@ -53,16 +54,21 @@ static void create_main_menu(GtkWidget *w);
 static void about_event_cb(GtkWidget* w, GdkEventAny* e, gpointer data);
 static void help_event_cb(GtkWidget* w, GdkEventAny* e, gpointer data);
 static  osso_context_t *osso_context = NULL;
+static gboolean key_press_cb(GtkWidget *, GdkEventKey *, HildonWindow *);
 #endif
 
+#ifdef MAEMO
+static HildonWindow *window;
+#else
+static GtkWidget  *window;
+#endif
+static int fullscreen = 0;
+
 int main(int argc, char *argv[])
 {
    /* Create needed variables */
 #ifdef MAEMO
    HildonProgram *program;
-   HildonWindow *window;
-#else
-   GtkWidget  *window;
 #endif
    GtkWidget *car_tab, *entry_tab, *data_tab, *care_tab, *trip_tab, *graph_tab;
    GtkWidget *car_label, *entry_label, *data_label, *care_label, *trip_label, *graph_label;
@@ -95,8 +101,8 @@ int main(int argc, char *argv[])
    car_tab = gtk_hbox_new(FALSE, 0);
    entry_tab = gtk_hbox_new(FALSE, 0);
    data_tab = gtk_vbox_new(TRUE, 0);
-   care_tab = gtk_vbox_new(TRUE, 0);
-   trip_tab = gtk_vbox_new(TRUE, 0);
+   care_tab = gtk_hbox_new(FALSE, 0);
+   trip_tab = gtk_hbox_new(FALSE, 0);
    graph_tab = gtk_vbox_new(TRUE, 0);
 
    car_label = gtk_label_new("Car");
@@ -126,6 +132,14 @@ int main(int argc, char *argv[])
    common_tab_updateUnits();
    common_tab_updateCars();
 
+#ifdef MAEMO
+   /* Add hardware button listener to application */
+   g_signal_connect(G_OBJECT(window),
+		   "key_press_event",
+		   G_CALLBACK(key_press_cb),
+		   NULL);
+#endif
+
    gtk_signal_connect(GTK_OBJECT(window),
                      "delete_event",
                      GTK_SIGNAL_FUNC(delete_event_cb),
@@ -157,8 +171,7 @@ delete_event_cb(GtkWidget* window, GdkEventAny* e, gpointer data)
 void
 help_event_cb(GtkWidget* window, GdkEventAny* e, gpointer data)
 {
-#ifdef GREGALE
-#else
+#if GTK_CHECK_VERSION (2,10,0)
    hildon_help_show(osso_context, "Applications_mileage_mainview", HILDON_HELP_SHOW_DIALOG);
 #endif
 }
@@ -201,18 +214,36 @@ create_main_menu(GtkWidget *w)
    GtkWidget  *item_close;
    GtkWidget  *item_help;
    GtkWidget  *item_about;
+#if GTK_CHECK_VERSION(2,10,0)
+   GtkWidget  *item_backup;
+   GtkWidget  *item_import;
+   GtkWidget  *item_export;
+#endif
 
    main_menu = gtk_menu_new();
 
    item_about = gtk_menu_item_new_with_label("About");
    item_help = gtk_menu_item_new_with_label("Help");
    item_close = gtk_menu_item_new_with_label("Close");
+#if GTK_CHECK_VERSION(2,10,0)
+   item_backup = gtk_menu_item_new_with_label("Backup via Email");
+   item_import = gtk_menu_item_new_with_label("Import");
+   item_export = gtk_menu_item_new_with_label("Export");
 
+   gtk_menu_append(main_menu, item_backup);
+   gtk_menu_append(main_menu, item_import);
+   gtk_menu_append(main_menu, item_export);
+#endif
    gtk_menu_append(main_menu, item_about);
    gtk_menu_append(main_menu, item_help);
    gtk_menu_append(main_menu, item_close);
 
    hildon_window_set_menu(HILDON_WINDOW(w), GTK_MENU(main_menu));
+#if GTK_CHECK_VERSION(2,10,0)
+   g_signal_connect(G_OBJECT(item_backup), "activate", GTK_SIGNAL_FUNC(car_backup_clicked_cb), NULL);
+   g_signal_connect(G_OBJECT(item_import), "activate", GTK_SIGNAL_FUNC(car_import_clicked_cb), NULL);
+   g_signal_connect(G_OBJECT(item_export), "activate", GTK_SIGNAL_FUNC(car_export_clicked_cb), NULL);
+#endif
    g_signal_connect(G_OBJECT(item_about), "activate", GTK_SIGNAL_FUNC(about_event_cb), NULL);
    g_signal_connect(G_OBJECT(item_help), "activate", GTK_SIGNAL_FUNC(help_event_cb), NULL);
    g_signal_connect(G_OBJECT(item_close), "activate", GTK_SIGNAL_FUNC(delete_event_cb), NULL);
@@ -227,5 +258,32 @@ mileage_get_osso()
 {
    return osso_context;
 }
+
+gboolean
+key_press_cb(GtkWidget * widget, GdkEventKey * event,
+		HildonWindow * w)
+{
+   GtkWindow *win;
+   switch (event->keyval)
+   {
+      case GDK_F6:
+	 win = GTK_WINDOW(window);
+	 if (fullscreen == 0)
+         {
+            fullscreen = 1;
+            gtk_window_fullscreen(win);
+         }
+	 else
+         {
+            fullscreen = 0;
+            gtk_window_unfullscreen(win);
+         }
+	 data_tab_fullscreen(fullscreen);
+	 care_tab_fullscreen(fullscreen);
+	 trip_tab_fullscreen(fullscreen);
+         return TRUE;
+   }
+   return FALSE;
+}
 #endif
 
